# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin


from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# PAYMENT FORM
class PaymentFormViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    PaymentFormViewSet
    -----------

    :action list: Return the payment form list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given payment form.
        `Profiles`: `superuser`, `staff`

    :action light: Return the payment form list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new payment form instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a payment form instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a payment form instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a payment form instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PaymentForm.objects.all()
    serializer_class = serializers.PaymentFormSerializer
    filter_backends = (filters.PaymentFormFilterBackend,)
    permission_classes = (permissions.PaymentFormPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a payment form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a payment form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PAYMENT INSTALLMENT
class PaymentInstallmentViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    PaymentInstallmentViewSet
    -----------

    :action list: Return the payment installment list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given payment installment.
        `Profiles`: `superuser`, `staff`

    :action light: Return the payment installment list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a payment installment instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PaymentInstallment.objects.all()
    serializer_class = serializers.PaymentInstallmentSerializer
    filter_backends = (filters.PaymentInstallmentFilterBackend,)
    permission_classes = (permissions.PaymentInstallmentPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a payment installment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a payment installment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
